/**
 * $Revision: 3023 $
 * $Date: 2005-11-02 18:00:15 -0300 (Wed, 02 Nov 2005) $
 *
 * Copyright (C) 2005 Jive Software. All rights reserved.
 *
 * This software is published under the terms of the GNU Public License (GPL),
 * a copy of which is included in this distribution.
 */

package org.jivesoftware.smackx.commands;

/**
 * Notes can be added to a command execution response. A note has to attributes,
 * one is the value or message an the other is the type of the note.
 * 
 * @author Gabriel Guardincerri
 * 
 */
public class AdHocCommandNote {

    private Type type;

    private String value;

    public AdHocCommandNote(Type type, String value) {
        this.type = type;
        this.value = value;
    }

    /**
     * Returns the value or message of the note.
     * 
     * @return the value or message of the note.
     */
    public String getValue() {
        return value;
    }

    /**
     * Return the type of the note.
     * 
     * @return the type of the note.
     */
    public Type getType() {
        return type;
    }

    /**
     * The types of the notes.
     * 
     * @author Gabriel Guardincerri
     * 
     */
    public enum Type {

        /**
         * The note is informational only. This is not really an exceptional
         * condition.
         */
        info,

        /**
         * The note indicates a warning. Possibly due to illogical (yet valid)
         * data.
         */
        warm,

        /**
         * The note indicates an error. The text should indicate the reason for
         * the error.
         */
        error
    }

}
